from html import escape
from re import compile as compiler, sub

# Constants
HTML_TAG_PATTERN = "<.*?>"
ESCAPE_CHARS = r"\*_`\["


# Clean HTML
async def clean_html(raw_html: str) -> str:
    cleaner = compiler(HTML_TAG_PATTERN)
    return sub(cleaner, "", raw_html)


# Escape Markdown
async def escape_markdown(text: str) -> str:
    return sub(r"([%s])" % ESCAPE_CHARS, r"\\\1", text)


# Mention in HTML
async def mention_html(name: str, user_id: int) -> str:
    return f'<a href="tg://user?id={user_id}">{escape(name)}</a>'


# Mention in Markdown
async def mention_markdown(name: str, user_id: int) -> str:
    escaped_name = await escape_markdown(name)
    return f"[{escaped_name}](tg://user?id={user_id})"


# Remove Markdown and HTML
async def remove_markdown_and_html(text: str) -> str:
    cleaned_html = await clean_html(text)
    return await clean_markdown(cleaned_html)


# Clean HTML Tags
async def clean_html_tags(text: str) -> str:
    tags = ["<code>", "</code>", "<b>", "</b>", "<i>", "</i>", "<u>", "</u>"]
    for tag in tags:
        text = text.replace(tag, "")
    return text


# Clean Markdown
async def clean_markdown(text: str) -> str:
    markdown_chars = ["`", "**", "__"]
    for char in markdown_chars:
        text = text.replace(char, "")
    return text



import asyncio
import typing
from logging import getLogger

try:
    import pyrogram
    from pyrogram import errors
except ImportError:
    import hydrogram as pyrogram
    from hydrogram import errors

LOGGER = getLogger(__name__)


def callback(
        self,
        data: typing.Union[str, list],
        is_bot: typing.Union[bool, bool] = False,
        is_user: typing.Union[bool, bool] = False,
        filtercb: typing.Union[pyrogram.filters.Filter] = None,
        *args,
        **kwargs,
):
    """
    ### `Client.on_cb("data")`

    - A decorator to Register Callback Quires in simple way and manage errors in that Function itself, alternative
    for `@hydrogram.Client.on_callback_query(hydrogram.filters.regex('^data.*'))`
    - Parameters: - data (str || list):
    - The callback query to be handled for a function

    - Is_bot (bool) **optional**:
        - If True, the command will only be executed if the Bot is Admin in the Chat, By Default, False

    - Is_user (bool) **optional**:
        - If True, the command will only be executed if the User is Admin in the Chat, By Default, False

    - Filter (`~hydrogram.filters`) **optional**:
        - hydrogram Filters, hope you know about this, for Advanced usage.
        Use `and` for exasperating filters.

    #### Example.
    Code-block:: python
        import hydrogram

        app = pyrogram.Client()

        @app.on_cmd("start")
        async def start(client, message):
            await message.reply_text(
            f" Hello {message.from_user.mention}",
            reply_markup=hydrogram.types.InlineKeyboardMarkup([[
                hydrogram.types.InlineKeyboardButton("Click Here",
                "data"
                )
            ]])
            )

        @app.on_cb("data")
        async def data(client, CallbackQuery):
        await CallbackQuery.answer("Hello: /", show_alert=True)
    """
    if filtercb:
        filtercb = pyrogram.filters.regex(f"^{data}.*") & args["filter"]
    else:
        filtercb = pyrogram.filters.regex(f"^{data}.*")

    def wrapper(func):
        async def decorator(abg: pyrogram.Client, q: pyrogram.types.CallbackQuery):
            if is_bot:
                me = await abg.get_chat_member(
                    q.message.chat.id, (await abg.get_me()).id
                )
                if me.status not in (
                        pyrogram.enums.ChatMemberStatus.OWNER,
                        pyrogram.enums.ChatMemberStatus.ADMINISTRATOR,
                ):
                    return await q.message.edit_text(
                        "I must be admin to execute this command."
                    )
            if is_user:
                try:
                    user = await q.message.chat.get_member(q.from_user.id)
                except Exception as e:
                    LOGGER.error("Error while fetching user status: " + str(e))
                    return
                if user.status not in (
                        pyrogram.enums.ChatMemberStatus.OWNER,
                        pyrogram.enums.ChatMemberStatus.ADMINISTRATOR,
                ):
                    return await q.message.edit_text(
                        "You must be admin to execute this command."
                    )
            try:
                await func(abg, q, *args, **kwargs)
            except errors.FloodWait as fw:
                LOGGER.warning(str(fw))
                await asyncio.sleep(fw.value)
                LOGGER.info(f"Sleeping for {fw.value}, Due to flood")
            except (errors.MessageIdInvalid, errors.MessageNotModified):
                pass
            except (errors.Forbidden, errors.ChatAdminRequired):
                LOGGER.warning(
                    f"Bot cannot write in chat: {q.message.chat.title} [{q.message.chat.id}] or need administration."
                )
                return await q.answer(
                    "I must be admin to execute this command.", show_alert=True
                )
            except Exception as e:
                return LOGGER.error(f"Error while executing command: {e}")

        self.add_handler(pyrogram.handlers.CallbackQueryHandler(decorator, filtercb))
        return decorator

    return wrapper


pyrogram.methods.Decorators.on_cb = callback



import asyncio
import typing
from logging import getLogger

from Abg.config import Config

try:
    import pyrogram
    from pyrogram import errors
except ImportError:
    import hydrogram as pyrogram
    from hydrogram import errors

HANDLER = Config.HANDLER
LOGGER = getLogger(__name__)


def command(
        self,
        cmd: typing.Union[str, list],
        is_disabled: typing.Union[bool, bool] = False,
        pm_only: typing.Union[bool, bool] = False,
        group_only: typing.Union[bool, bool] = False,
        self_admin: typing.Union[bool, bool] = False,
        self_only: typing.Union[bool, bool] = False,
        handler: typing.Optional[list] = None,
        filtercmd: typing.Union[pyrogram.filters.Filter, pyrogram.filters.Filter] = None,
        *args,
        **kwargs,
):
    """
    ### `@Client.on_cmd` - A Decorator to Register Commands in a simple way and manage errors in that Function itself,
    alternative for `@hydrogram.Client.on_message(hydrogram.filters.command('command'))`
    - Parameters: - cmd (str ||
    list): - The command to be handled for a function

    - group_only (bool) **optional**:
        - If True, the command will only executed in Groups only, By Default, False.

    - pm_only (bool) **optional**:
        - If True, the command will only executed in Private Messages only, By Default, False.

    - self_only (bool) **optional**:
        - If True, the command will only execute if used by Self only, By Default False.

    - handler (list) **optional**:
        - If set, the command will be handled by the specified Handler, By Default `Config.HANDLERS`.

    - self_admin (bool) **optional**:
        - If True, the command will only be executed if the Bot is Admin in the Chat, By Default, False

    - filtercmd (`~hydrogram.filters`) **optional**:
        - hydrogram Filters, hope you know about this, for Advanced usage.
        Use `and` for seaperating filters.

    #### Example.
    Code-block:: python
        import hydrogram

        App = hydrogram.Client(.)

        @app.on_cmd("start", is_disabled=False, group_only=False, pm_only=False,
        self_admin=False, self_only=False, filtercmd=hydrogram.filters.chat("777000")
        and hydrogram.filters.text)
        async def start(abg: Client, message):
            await message.reply_text(f" Hello {message.from_user.mention}")
    """
    if handler is None:
        handler = HANDLER
    if filtercmd:
        if self_only:
            filtercmd = (
                    pyrogram.filters.command(cmd, prefixes=handler)
                    & filtercmd
                    & pyrogram.filters.me
            )
        else:
            filtercmd = (
                    pyrogram.filters.command(cmd, prefixes=handler)
                    & filtercmd
                    & pyrogram.filters.me
            )
    else:
        if self_only:
            filtercmd = (
                    pyrogram.filters.command(cmd, prefixes=handler) & pyrogram.filters.me
            )
        else:
            filtercmd = pyrogram.filters.command(cmd, prefixes=handler)

    def wrapper(func):
        async def decorator(abg: pyrogram.Client, message: pyrogram.types.Message):
            if is_disabled:
                return await message.reply_text(
                    "This command is disabled by the Admins."
                )
            if group_only:
                if message.chat.type != pyrogram.enums.ChatType.SUPERGROUP or message.chat.type != pyrogram.enums.ChatType.GROUP:
                    return await message.reply_text(
                        "This command can be used in supergroups only."
                    )
            if self_admin:
                me = await abg.get_chat_member(message.chat.id, (await abg.get_me()).id)
                if me.status not in (
                        pyrogram.enums.ChatMemberStatus.OWNER,
                        pyrogram.enums.ChatMemberStatus.ADMINISTRATOR,
                ):
                    return await message.reply_text(
                        "I must be admin to execute this command."
                    )

            if pm_only and message.chat.type != pyrogram.enums.ChatType.PRIVATE:
                return await message.reply_text("This command can be used in PM only.")
            try:
                await func(abg, message, *args, **kwargs)
            except pyrogram.StopPropagation:
                raise pyrogram.StopPropagation  # Stop Propagation
            except pyrogram.ContinuePropagation:
                pass  # Do nothing
            except errors.FloodWait as fw:
                LOGGER.warning("Sleeping for {fw.value}, Due to flood")
                await asyncio.sleep(fw.value)
            except (errors.Forbidden, errors.SlowmodeWait):
                LOGGER.warning(f"Forbidden : {message.chat.title} [{message.chat.id}] doesn't have write permission.")
                return  # await message.chat.leave()
            except Exception as e:
                return LOGGER.error(f"Error while executing command: {e}")

        self.add_handler(
            pyrogram.handlers.MessageHandler(callback=decorator, filters=filtercmd)
        )
        return decorator

    return wrapper


pyrogram.methods.Decorators.on_cmd = command



def get_readable_time(seconds: int) -> str:
    count = 0
    ping_time = ""
    time_list = []
    time_suffix_list = ["Sec.", "Min.", "Hour", "Day", "Week", "Month", "Year"]
    while count < 4:
        count += 1
        remainder, result = divmod(seconds, 60) if count < 3 else divmod(seconds, 24)
        if seconds == 0 and remainder == 0:
            break
        time_list.append(int(result))
        seconds = int(remainder)
    for i in range(len(time_list)):
        time_list[i] = str(time_list[i]) + time_suffix_list[i]
    if len(time_list) == 4:
        ping_time += f"{time_list.pop()}, "
    time_list.reverse()
    ping_time += ":".join(time_list)
    return ping_time


async def convert_seconds_to_minutes(seconds: int):
    seconds = int(seconds)
    seconds %= 24 * 3600
    seconds %= 3600
    minutes = seconds // 60
    seconds %= 60
    return "%02d:%02d" % (minutes, seconds)


SIZE_UNITS = ["B", "KB", "MB", "GB", "TB", "PB"]


def get_readable_file_size(size_in_bytes) -> str:
    if size_in_bytes is None:
        return "0B"
    index = 0
    while size_in_bytes >= 1024:
        size_in_bytes /= 1024
        index += 1
    try:
        return f"{round(size_in_bytes, 2)}{SIZE_UNITS[index]}"
    except IndexError:
        return "File too large"


def get_readable_bitrate(bitrate_kbps):
    if bitrate_kbps > 10000:
        bitrate = str(round(bitrate_kbps / 1000, 2)) + " " + "mbps"
    else:
        bitrate = str(round(bitrate_kbps, 2)) + " " + "kbps"

    return bitrate



import logging
from datetime import datetime

from .__version__ import __version__

__copyright__ = (
    f"Copyright 2023 - {datetime.now().year} Abishnoi69 <github.com/Abishnoi69>"
)

log = logging.getLogger(__name__)
log.info(f"Version: {__version__}\nCopyright: {__copyright__}")

from .patch import *  # noqa



import contextlib
from functools import partial, wraps
from logging import getLogger
from typing import Union

from cachetools import TTLCache

from Abg.config import Config

LOGGER = getLogger(__name__)
ANON = TTLCache(maxsize=250, ttl=30)
DEVS = Config.DEVS

try:
    import pyrogram
    from pyrogram import errors
except ImportError:
    import hydrogram as pyrogram
    from hydrogram import errors


async def anonymous_admin_verification(
        self, callback: pyrogram.types.CallbackQuery
):
    if int(
            f"{callback.message.chat.id}{callback.data.split('.')[1]}"
    ) not in set(ANON.keys()):
        try:
            await callback.message.edit_text("Button has been expired")
        except errors.RPCError:
            with contextlib.suppress(errors.RPCError):
                await callback.message.delete()
        return
    cb = ANON.pop(
        int(f"{callback.message.chat.id}{callback.data.split('.')[1]}")
    )
    member = await callback.message.chat.get_member(callback.from_user.id)
    if member.status not in (
            pyrogram.enums.ChatMemberStatus.OWNER,
            pyrogram.enums.ChatMemberStatus.ADMINISTRATOR,
    ):
        return await callback.answer(
            "You need to be an admin to do this", show_alert=True
        )
    permission = cb[2]
    if getattr(member.privileges, permission) is True:
        try:
            await callback.message.delete()
            await cb[1](self, cb[0])
        except pyrogram.errors.exceptions.forbidden_403.ChatAdminRequired:
            return await callback.message.edit_text(
                "I must be an admin to execute this command",
            )
        except BaseException as e:
            LOGGER.error(f"Error Found in anonymous_admin_verification:{e}")
            return await callback.message.edit_text("An error occurred.")
    else:
        return await callback.message.edit_text(f"You don't have permission to {permission}.")


def adminsOnly(
        self,
        permissions: Union[str] = None,
        is_bot: bool = False,
        is_user: bool = False,
        is_both: bool = False,
        only_owner: bool = False,
        only_dev: bool = False,
        allow_pm: bool = True,
        allow_channel: bool = True,
):
    """Check for permission level to perform some operations

    Args:
        self:
        permissions (str, optional): permission type to check. Defaults to None.
        is_bot (bool, optional): If bot can perform the action. Defaults to False.
        is_user (bool, optional): If user can perform the action. Defaults to False.
        is_both (bool, optional): If both user and bot can perform the action. Defaults to False.
        only_owner (bool, optional): If only owner can perform the action. Defaults to False. (It's Chat Owner)
        only_dev (bool, optional): if only dev users can perform the operation. Defaults to False.
        allow_channel (bool, optional): If the command can be used in channels. Defaults to True.
        allow_pm (bool, optional): If the command can be used in PM. Defaults to True.
    """

    def decorator(func):
        @wraps(func)
        async def wrapper(
                abg: pyrogram.Client, message: Union[pyrogram.types.CallbackQuery, pyrogram.types.Message], *args,
                **kwargs
        ):
            if isinstance(message, pyrogram.types.CallbackQuery):
                sender = partial(message.answer, show_alert=True)
                msg = message.message
                chat = message.message.chat
            else:
                sender = message.reply_text
                msg = message
                chat = message.chat

            if msg.chat.type == pyrogram.enums.ChatType.PRIVATE and not (only_dev or only_owner):
                if allow_pm:
                    return await func(abg, message, *args, *kwargs)
                return await sender("This command can't be used in PM.")

            if msg.chat.type == pyrogram.enums.ChatType.CHANNEL and not (only_dev or only_owner):
                if allow_channel:
                    return await func(abg, message, *args, *kwargs)
                return await sender("This command can't be used in channels.")

            if not msg.from_user:
                ANON[int(f"{msg.chat.id}{msg.id}")] = (msg, func, permissions)
                keyboard = pyrogram.types.InlineKeyboardMarkup(
                    [
                        [
                            pyrogram.types.InlineKeyboardButton(
                                text="✅ Verify Admin",
                                callback_data=f"anon.{msg.id}",
                            ),
                        ]
                    ]
                )
                return await msg.reply_text(
                    "Please verify that you are an admin to perform this action.",
                    reply_markup=keyboard,
                )

            try:
                bot = await chat.get_member(abg.me.id)
                user = await chat.get_member(message.from_user.id)
            except pyrogram.errors.exceptions.bad_request_400.ChatAdminRequired:
                return await sender(f"I must be admin to execute this command.")
            except pyrogram.errors.exceptions.forbidden_403.ChatAdminRequired:
                return await sender(f"I must be admin to execute this command.")
            except errors.UserNotParticipant:
                return await sender(
                    f"User: {message.from_user.first_name} not member of this chat."
                )
            except Exception as e:
                LOGGER.error(f"Error Found in adminsOnly:{e}")
                return await sender(f"An error occurred: {e}")

            if only_dev:
                if msg.from_user.id in DEVS:
                    return await func(abg, message, *args, **kwargs)
                else:
                    return await sender(
                        "Only devs can perform this action.",
                    )

            if msg.from_user.id in DEVS:
                return await func(abg, message, *args, **kwargs)

            if only_owner:
                if user.status == pyrogram.enums.ChatMemberStatus.OWNER:
                    return await func(abg, message, *args, **kwargs)
                else:
                    return await sender("Only chat owner can perform this action")

            if permissions:
                no_permission = permissions
                if permissions == "can_promote_members":
                    no_permission = "promote members"
                elif permissions == "can_change_info":
                    no_permission = "change chat info"
                elif permissions == "can_pin_messages":
                    no_permission = "pin messages"
                elif permissions == "can_invite_users":
                    no_permission = "invite users"
                elif permissions == "can_restrict_members":
                    no_permission = "restrict members"
                elif permissions == "can_delete_messages":
                    no_permission = "delete messages"
                elif permissions == "can_manage_chat":
                    no_permission = "manage chat"
                elif permissions == "can_manage_video_chats":
                    no_permission = "manage video chats"
                elif permissions == "can_post_messages":
                    no_permission = "post messages"
                elif permissions == "can_edit_messages":
                    no_permission = "edit messages"
                elif permissions == "can_manage_topics":
                    no_permission = "manage chat topics"
                elif permissions == "is_anonymous":
                    no_permission = "anonymous"
                if is_bot:
                    if bot.status != pyrogram.enums.ChatMemberStatus.ADMINISTRATOR:
                        return await sender("I must be admin to execute this command.")

                    if getattr(bot.privileges, permissions) is True:
                        return await func(abg, message, *args, **kwargs)
                    else:
                        return await sender(
                            f"I don't have permission to {no_permission}."
                        )
                if is_user:
                    if user.status not in [
                        pyrogram.enums.ChatMemberStatus.ADMINISTRATOR,
                        pyrogram.enums.ChatMemberStatus.OWNER,
                    ]:
                        return await sender(
                            "You must be an admin to use this command."
                        )

                    if getattr(user.privileges, permissions) is True:
                        return await func(abg, message, *args, **kwargs)
                    else:
                        return await sender(
                            f"You don't have permission to {no_permission}."
                        )
                if is_both:
                    if bot.status != pyrogram.enums.ChatMemberStatus.ADMINISTRATOR:
                        return await sender("I must be admin to execute this command.")

                    if getattr(bot.privileges, permissions) is True:
                        pass
                    else:
                        return await sender(
                            f"I don't have permission to {no_permission}."
                        )

                    if user.status not in [
                        pyrogram.enums.ChatMemberStatus.ADMINISTRATOR,
                        pyrogram.enums.ChatMemberStatus.OWNER,
                    ]:
                        return await sender(
                            "You must be an admin to use this command."
                        )

                    if getattr(user.privileges, permissions) is True:
                        pass
                    else:
                        return await sender(
                            f"You don't have permission to {no_permission}."
                        )
                    return await func(abg, message, *args, **kwargs)
                else:
                    if is_bot:
                        if bot.status == pyrogram.enums.ChatMemberStatus.ADMINISTRATOR:
                            return await func(abg, message, *args, **kwargs)
                        else:
                            return await sender(
                                "I must be admin to execute this command."
                            )
                    elif is_user:
                        if user.status in [
                            pyrogram.enums.ChatMemberStatus.ADMINISTRATOR,
                            pyrogram.enums.ChatMemberStatus.OWNER,
                        ]:
                            return await func(abg, message, *args, **kwargs)
                        elif msg.from_user.id in DEVS:
                            return await func(abg, message, *args, **kwargs)
                        else:
                            return await sender(
                                "You must be an admin to use this command."
                            )
                    elif is_both:
                        if bot.status == pyrogram.enums.ChatMemberStatus.ADMINISTRATOR:
                            pass
                        else:
                            return await sender(
                                "I must be admin to execute this command."
                            )

                        if user.status in [
                            pyrogram.enums.ChatMemberStatus.ADMINISTRATOR,
                            pyrogram.enums.ChatMemberStatus.OWNER,
                        ]:
                            pass
                        elif msg.from_user.id in DEVS:
                            pass
                        else:
                            return await sender(
                                "You must be an admin to use this command"
                            )
                        return await func(abg, message, *args, **kwargs)

        self.add_handler(
            pyrogram.handlers.CallbackQueryHandler(
                anonymous_admin_verification,
                pyrogram.filters.regex("^anon."),
            ),
        )
        return wrapper

    return decorator


pyrogram.methods.Decorators.adminsOnly = adminsOnly



import os
from typing import Dict

from setuptools import setup, find_packages

base_path = os.path.abspath(os.path.dirname(__file__))

about: Dict = {}
with open(
    os.path.join(
        base_path,
        'Abg',
        '__version__.py',
    ), encoding='utf-8',
) as f:
    exec(f.read(), about)

DESCRIPTION = 'add-on for Pyrogram || add-on for Hydrogram || Telegram bot helpers || Easy botting'
with open("README.md", encoding="utf8") as readme:
    long_description = readme.read()

setup(
    name="Abg",
    version=about["__version__"],
    author="Abishnoi (Ashok Bishnoi)",
    author_email="<abishnoi69@outlook.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=[
        'python-dotenv~=1.0.1',
        'cachetools~=5.4.0',
        'pytz>=2024.1'
    ],
    extras_require={
        'hydrogram': [
            'hydrogram~=0.2.0',
        ],
    },
    keywords="add-on bots telegram bot hydrogram pyrogram",
    url="https://github.com/Abishnoi69/Abg",
    download_url="https://github.com/Abishnoi69/Abg/releases/latest",
    license="MIT",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: Implementation",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Internet",
        "Topic :: Communications",
        "Topic :: Communications :: Chat",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
    ],
    project_urls={
        "Tracker": "https://github.com/Abishnoi69/Abg/issues",
        "Community": "https://t.me/Abgpy",
        "Source": "https://github.com/Abishnoi69/Abg",
        "Documentation": "https://abg.abishnoi.me/",
    },
    python_requires=">=3.8",
)



from .inline_keyboard import InlineButton, InlineKeyboard
from .inline_pagination_keyboard import InlineButton as PaginationInlineButton
from .reply_keyboard import ReplyButton

__all__ = ["InlineButton", "InlineKeyboard", "PaginationInlineButton", "ReplyButton"]



from os import getenv

from dotenv import load_dotenv

load_dotenv()


class Config(object):
    HANDLER = getenv("HANDLER", "/ ! + . $ #").split()
    devs_env = getenv("DEVS")
    DEVS = list(map(int, devs_env.split())) if devs_env else []
